'use strict'

const RepositoryCredential = use('App/Models/RepositoryCredential');
const Webhook = use('App/Models/Webhook');

class GitlabScriptController {
    async handleFourOhFour({response}) {
        return response.bash(404, null, [{message: 'No Such Endpoint'}]);
    }

    async start({request, response}) {
        const BASH = use('bash');
        const bash = new BASH();
        const req = this._getReq(request);
        bash.echo(`Notified Hermes that CI/CD Job is Starting for Project "${req.project}" Environment "${req.environment}"`);
        /**
         * Push notification via websocket
         */
        return response.bash(200, bash.toString());
    }

    async end({request, response}) {
        const BASH = use('bash');
        const bash = new BASH();
        const req = this._getReq(request);
        bash.echo(`Notified Hermes that CI/CD Job is Ending for Project "${req.project}" Environment "${req.environment}"`);
        /**
         * Push notification via websocket
         */
        return response.bash(200, bash.toString());
    }

    async deploy({request, response}) {
        const BASH = use('bash');
        const bash = new BASH();
        const req = this._getReq(request);
        bash.runningPipelineCheck(request, req);
        bash.echo(`Deploying Project "${req.project}" to Environment "${req.environment}"`);
        /**
         * Retrieve all active credentials for this project + environment
         */
        try {
            const credentials = await RepositoryCredential.query().where('gitlab_repository_id', req.project).where('environment', req.environment).where('active', true).fetch();
            if (Array.isArray(credentials.rows) && credentials.rows.length > 0) {
                for (let i = 0; i < credentials.rows.length; i++) {
                    const row = credentials.rows[i];
                    const methodKey = `deploy${this._changeMethodCapitalization(row.method)}`;
                    if ('function' == typeof(bash[methodKey])) {
                        this._exportCredentials(row, bash);
                        bash[methodKey](row);
                        this._obfuscateSensativeCredentials(bash);
                        try {
                            const webhooks = await Webhook.query().where('active', true).where(function() {
                                this.where('repository_credentials_id', row.id).orWhereNull('repository_credentials_id')
                            }).fetch();
                            if (Array.isArray(webhooks.rows) && webhooks.rows.length > 0) {
                                bash.echo('Firing Webhooks');
                                for (let wi = 0; wi < webhooks.rows.length; wi++) {
                                    const hook = webhooks.rows[wi];
                                    bash.webhook(hook);
                                }
                            }
                        }
                        catch (error) {
                            console.warn(`Failed to retrieve webhooks for credentials ${row.id}: ${error.toString()}`);
                            console.warn(error);
                        }
                        this._cleanupCredentials(bash);
                        bash.echo(`Finished Deploying to Credentials #${row.id}`);
                    }
                    else {
                        bash.echo(`Deployment failed for credentials #${row.id}. Hermes is not yet able to deploy using method "${row.method}".`);
                        console.warn(`Function "${methodKey}" does not exist`);
                    }
                }
            }
            else {
                bash.echo(`No active credentials found for project #${req.project} environment "${req.environment}"`);
            }
        }
        catch (error) {
            bash.clear();
            bash.echo('Hermes encountered an unrecoverable error. Please check with the Hermes administrator for more information.');
            console.warn(`Unable to retrieve credentials due to error: ${error.toString()}`);
            console.warn(error);
            return response.bash(500, bash.toString());
        }
        bash.echo(`Finished Deploying Project "${req.project}" to Environment "${req.environment}"`);
        /**
         * Push notification via websocket
         */
        return response.bash(200, bash.toString());
    }

    async checkPipeline({request, response}) {
        const qs = require('qs');
        const BASH = use('bash');
        const bash = new BASH();
        const {validate} = use('Validator');
        const rules = {
            ref: 'required|string',
            pipeline: 'required|integer',
        };
        const messages = {
            'ref.required': 'The commit reference name is required',
            'ref.string': 'Please pass a valid value for the commit reference name',
            'pipeline.required': 'The current pipeline ID is required',
            'pipeline.integer': 'The current pipeline ID must be an integer',
        }
        const validation = await validate(request.all(), rules, messages);
        if (validation.fails()) {
            bash.echo('There is information missing in the request');
            bash.send('exit 400');
            return response.bash(400, bash.toString(), validation.messages());
        }
        const req = this._getReq(request);
        const gitlab = this._getGitlab(request);
        // bash.echo(`Searching for pipelines older than "${req.pipeline}" running for commit reference name "${req.ref}" in Project "${req.project}"`);
        const statuses = ['pending', 'running'];
        const older_pipelines = [];
        for (let i = 0; i < statuses.length; i++) {
            const status = statuses[i];
            const uri = `/projects/${req.project}/pipelines?${qs.stringify({
                status,
                ref: req.ref,
            })}`;
            try {
                const {data} = await gitlab.get(uri);
                if (Array.isArray(data) && data.length > 0) {
                    for (let pi = 0; pi < data.length; pi++) {
                        const pipeline = data[pi];
                        if (pipeline.id < req.pipeline) {
                            older_pipelines.push(pipeline);
                        }
                    }
                }
            }
            catch (error) {
                bash.echo('There was an error retrieving pipelines. Script Stopped.');
                bash.send('exit 500');
                return response.bash(500, bash.toString(), [{message: error.toString()}]);
            }
        }
        if (older_pipelines.length > 0) {
            bash.echo(`There are ${older_pipelines.length} previously enqueued or running pipelines. Checking again in 30 seconds.`);
            bash.send('sleep 30s');
            bash.runningPipelineCheck(request, req);
        }
        else {
            bash.echo('There are no previously enqueued or running pipelines. Continuing.');
        }
        return response.bash(200, bash.toString());
    }

    _exportCredentials(c, bash) {
        bash.echo(`Setting Environmental Variables for Credentials #${c.id}`);
        bash.export('DEPLOY_USERNAME', c.user);
        bash.export('DEPLOY_HOST', c.host);
        bash.export('DEPLOY_PORT', c.port);
        bash.export('DEPLOY_AUTH_METHOD', c.authmethod);
        bash.export('DEPLOY_PASSWORD', c.password);
        bash.export('DEPLOY_KEY', c.pemkey);
        bash.export('DEPLOY_PATH', c.path);
        bash.export('DEPLOY_METHOD', c.method);
        bash.export('DEPLOY_REMOTE_OWNER', c.suser);
        bash.export('DEPLOY_REMOTE_GROUP', c.sgroup);
        if ('pem' == c.authmethod) {
            bash.send(`touch /tmp/deploykey.pem`);
            bash.send('echo "$DEPLOY_KEY" > /tmp/deploykey.pem');
            bash.send('chmod 400 /tmp/deploykey.pem');
        }
        bash.echo(`Environmental Variables for Credentials #${c.id} have been set`);
    }

    _obfuscateSensativeCredentials(bash) {
        bash.echo(`Obfuscating Sensative Variables`);
        bash.export('DEPLOY_PASSWORD', 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX');
        bash.export('DEPLOY_KEY', 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX');
        bash.send(`rm /tmp/deploykey.pem || true`);
        bash.echo(`Sensative Variables have been Obfuscated`);
    }

    _cleanupCredentials(bash) {
        bash.echo(`Cleaning Up Credentials`);
        bash.send('unset DEPLOY_USERNAME');
        bash.send('unset DEPLOY_HOST');
        bash.send('unset DEPLOY_PORT');
        bash.send('unset DEPLOY_AUTH_METHOD');
        bash.send('unset DEPLOY_PASSWORD');
        bash.send('unset DEPLOY_KEY');
        bash.send('unset DEPLOY_PATH');
        bash.send('unset DEPLOY_METHOD');
        bash.send('unset DEPLOY_REMOTE_OWNER');
        bash.send('unset DEPLOY_REMOTE_GROUP');
        bash.send(`rm /tmp/deploykey.pem || true`);
        bash.echo(`Credentials Cleaned Up`);
    }

    _getReq(request) {
        const {sanitize} = use('Validator');
        const transformations = {
            project: 'to_int',
            environment: 'trim',
            ref: 'trim',
            pipeline: 'to_int',
        }
        const req = sanitize(request.all(), transformations);
        return req;
    }

    _changeMethodCapitalization(string) {
        return string[0].toUpperCase() + string.slice(1);
    }

    _getGitlab(request) {
        const gitlab = use('gitlab')();
        const Config = use('Config');
        gitlab.setAuthMethod('pat');
        const auth = request.header('authorization');
        const b64 = auth.substr(6);
        const buff = Buffer.from(b64, 'base64');
        const basic = buff.toString('ascii');
        const authparts = basic.split(':');
        if ('gitlab-ci-token' == authparts[0]) {
            gitlab.setToken(Config.get('gitlab.token'));
        }
        else {
            gitlab.setToken(authparts[1]);
        }
        return gitlab;
    }
}

module.exports = GitlabScriptController
