'use strict'

const compareVersions = require('compare-versions');
const { validate, sanitize} = use('Validator');
const RepositoryCredential = use('App/Models/RepositoryCredential');
const Config = use('Config');

class RepositoryController {
    async info({params, response, auth}) {
        const user = await auth.getUser();
        const gitlab = use('gitlab')();
        gitlab.setOauth(user.oauth);
        try {
            const {data} = await gitlab.get(`/projects/${params.id}`);
            return response.api(200, data);
        }
        catch (error) {
            return gitlab.handleError(error, response);
        }
    }

    async readme({params, response, auth}) {
        const user = await auth.getUser();
        const gitlab = use('gitlab')();
        gitlab.setOauth(user.oauth);
        try {
            const {data} = await gitlab.get(`/projects/${params.id}`);
            if (null == data.readme_url) {
                return response.api(200, null);
            }
            const lastIndex = data.readme_url.lastIndexOf('/');
            if (lastIndex == -1) {
                return response.api(200, null);
            }
            const filename = data.readme_url.substring(lastIndex + 1);
            const url = `/projects/${params.id}/repository/files/${filename}/raw?ref=master`;
            const fr = await gitlab.get(url);
            return response.api(200, fr.data);
        }
        catch (error) {
            return gitlab.handleError(error, response);
        }
    }

    async badges({params, response, auth}) {
        const user = await auth.getUser();
        const gitlab = use('gitlab')();
        gitlab.setOauth(user.oauth);
        try {
            const {data} = await gitlab.get(`/projects/${params.id}`);
            let release_in_testing = null;
            const vr = await gitlab.get(`/projects/${params.id}/variables`);
            for (let i = 0; i < vr.data.length; i++) {
                const variable = vr.data[i];
                if (variable.key == 'RELEASE_IN_TESTING') {
                    release_in_testing = variable.value;
                }
            }
            const ret = {
                default: null,
                stable: null,
                testing: null,
            };
            const pr = await gitlab.get(`/projects/${params.id}/pipelines`);
            let stable = 'v0.0.0';
            for (let i = 0; i < pr.data.length; i++) {
                const pipeline = pr.data[i];
                if (pipeline.ref == data.default_branch && null == ret.default) {
                    ret.default = pipeline;
                }
                if (pipeline.ref.includes('v') && compareVersions(pipeline.ref, stable) > 0) {
                    ret.stable = pipeline;
                    stable = pipeline.ref;
                }
                if (pipeline.ref == release_in_testing && null == ret.testing) {
                    ret.testing = pipeline;
                }
            }
            return response.api(200, ret);
        }
        catch (error) {
            if ('undefined' !== typeof(error.response) && 403 == error.response.status) {
                return response.api(200, {
                    default: null,
                    stable: null,
                    testing: null,
                });
            }
            return gitlab.handleError(error, response);
        }
    }

    async credentials({params, request, response, auth}) {
        const Encryption = use('Encryption')
        const query = RepositoryCredential.query().where('gitlab_repository_id', params.id);
        const search = request.input('search');
        const options = request.input('options');
        let page = 1;
        let itemsPerPage = 10;
        if ('string' === typeof(search) && search.length > 0) {
            query.where(function() {
              this.where('environment', 'like', `%${search}%`)
            });
        }
        if ('object' == typeof(options)) {
            page = options.page;
            itemsPerPage = options.itemsPerPage;
            if (Array.isArray(options.sortBy)) {
                for (let i = 0; i < options.sortBy.length; i++) {
                    const key = options.sortBy[i];
                    const direction = (Array.isArray(options.sortDesc) && options.sortDesc.indexOf(key) > -1) ? 'desc' : 'asc';
                    query.orderBy(key, direction);
                }
            }
            if (Array.isArray(options.groupBy)) {
                for (let i = 0; i < options.groupBy.length; i++) {
                    const key = options.groupBy[i];
                    query.groupBy(key);
                }
                if (['pg'].indexOf(Config.get('database.connection')) > -1) {
                    query.groupBy('id');
                }
            }
        }
        const results = await query.paginate(page, itemsPerPage);
        const ret = results.toJSON();
        ret.data = ret.data.map((item) => {
            const ni = {};
            ni.id = item.id;
            ni.active = (true == item.active);
            ni.environment = item.environment;
            ni.host = Encryption.decrypt(item.host);
            ni.method = item.method;
            ni.authmethod = item.authmethod;
            return ni;
        })
        return response.api(200, ret);
        // console.log(ret);
        // return response.api(418, null, []);
    }

    async webhooks({params, response, auth}) {
        return response.api(418, null, []);
    }

    async getCredentials({ response, params }) {
        try {
            const item = await RepositoryCredential.findOrFail(params.cid);
            if (item.gitlab_repository_id !== parseInt(params.id)) {
                return response.api(404, null, [{message: 'The specified repository credentials do not exist for this repository'}]);
            }
            const ret = item.toJSON();
            ret.password = '';
            ret.pemkey = '';
            return response.api(200, ret);
        }
        catch (error) {
            if (true == Config.get('app.debug')) {
                console.warn(`Error retrieving credentials: ${error.toString()}`);
                console.warn(error);
            }
            return response.api(500, null, [{message: 'An error occured while attempting to update the specified Repository Credentials'}]);
        }
    }

    async storeCredentials({ request, response, params }) {
        const rules = {
            active: 'required|boolean',
            environment: 'required|string',
            user: 'required_when:method,rsync|string',
            host: 'required_when:method,rsync|string',
            port: 'required_when:method,rsync|number|min:1|max:65535',
            authmethod: 'required_when:method,rsync|in:password,pem',
            password: 'required_when:authmethod:password|string',
            pemkey: 'required_when:authmethod:pem|string',
            method: 'required|in:none,rsync',
            path: 'required_when:method,rsync|string',
            suser: 'required_when:method,rsync|string',
            sgroup: 'required_when:method,rsync|string',
            predeploy: 'string',
            postdeploy: 'string',
        };
        if (parseInt(params.cid) > 0) {
            rules.password = 'string';
            rules.pemkey = 'string';
        }
        const transformations = {
            active: 'toBoolean',
            port: 'to_int',
            environment: 'trim',
            user: 'trim',
            host: 'trim',
            password: 'trim',
            pemkey: 'trim',
            path: 'trim',
            suser: 'trim',
            sgroup: 'trim',
            predeploy: 'trim',
            postdeploy: 'trim',
        }
        const messages = {
            'active.required': 'You must pass an activation state',
            'active.boolean': 'The activation state must be a boolean',
            'environment.required': 'The name of the environment is required',
            'environment.string': 'The name of the environment should be a string',
            'user.required_when': 'User is required',
            'host.required_when': 'Host is required',
            'port.required_when': 'Port is required',
            'authmethod.required_when': 'You must specify an authenticaation method',
            'authmethod.in': 'Available Authentication Methods are: Password and PEM Key',
            'password.required_when': 'You must provide a password',
            'password.string': 'Please provide a valid value for password',
            'pemkey.required_when': 'You must provide a PEM Key',
            'pemkey.string': 'Please provide a valid PEM Key',
            'method.required': 'Please specify a valid deployment method',
            'method.in': 'Valid Deployment Methods are None and RSYNC Push',
            'path.required_when': 'The remote deployment path is required',
            'path.string': 'The remote deployment path is required',
            'suser.required_when': 'The remote owning user is required',
            'suser.string': 'The remote owning user is required',
            'sgroup.required_when': 'The remote owning group is required',
            'sgroup.string': 'The remote owning group is required',
            'predeploy.string': 'Please provide a valid value for Pre-Deployment Commands',
            'postdeploy.string': 'Please provide a valid value for Post-Deployment Commands',
        }
        const validation = await validate(request.all(), rules, messages);
        if (validation.fails()) {
            return response.api(400, null, validation.messages());
        }
        const req = sanitize(request.all(), transformations);
        const stringify = ['environment', 'user', 'host', 'authmethod', 'password', 'pemkey', 'method', 'path', 'suser', 'sgroup', 'predeploy', 'postdeploy'];
        for (let i = 0; i < stringify.length; i++) {
            const prop = stringify[i];
            req[prop] = (null == req[prop]) ? '' : req[prop];
        }
        let item;
        if (!params.cid) {
            item = new RepositoryCredential();
        }
        else {
            try {
                item = await RepositoryCredential.findOrFail(params.cid);
            }
            catch (error) {
                return response.api(500, null, [{message: 'An error occured while attempting to update the specified Repository Credentials'}]);
            }
        }
        item.environment = req.environment;
        item.active = req.active;
        item.user = req.user;
        item.host = req.host;
        item.port = req.port;
        item.authmethod = req.authmethod;
        if (!params.cid || req.password.length > 0) {
            item.password = req.password;
        }
        if (!params.cid || req.pemkey.length > 0) {
            item.pemkey = req.pemkey;
        }
        item.method = req.method;
        item.path = req.path;
        item.suser = req.suser;
        item.sgroup = req.sgroup;
        item.predeploy = req.predeploy;
        item.postdeploy = req.postdeploy;
        item.additional = req.additional;
        item.gitlab_repository_id = parseInt(params.id);
        try {
            await item.save();
            return response.api(201, item);
        }
        catch (error) {
            if (Config.get('app.debug')) {
                console.warn(`Error saving Repository Credentials: ${error.toString()}`);
                console.warn(error);
            }
            return response.api(500, null, [{message: 'An error occured while trying to save this item. Please check with the system administrator.'}]);
        }
        return response.api(418, null, [{message: 'Debugging'}]);
    }

    async storeCredentialActivation({ request, response, params }) {
        const rules = {
            state: 'required|boolean',
        };
        const transformations = {
            state: 'toBoolean',
        }
        const messages = {
            'state.required': 'You must pass an activation state',
            'state.boolean': 'The activation state must be a boolean',
        }
        const validation = await validate(request.all(), rules, messages);
        if (validation.fails()) {
            return response.api(400, null, validation.messages());
        }
        const req = sanitize(request.all(), transformations);
        const stringify = ['environment', 'user', 'host', 'authmethod', 'password', 'pemkey', 'method', 'path', 'suser', 'sgroup', 'predeploy', 'postdeploy'];
        for (let i = 0; i < stringify.length; i++) {
            const prop = stringify[i];
            req[prop] = (null == req[prop]) ? '' : req[prop];
        }
        let item;
        try {
            item = await RepositoryCredential.findOrFail(params.cid);
        }
        catch (error) {
            return response.api(500, null, [{message: 'An error occured while attempting to update the specified Repository Credentials'}]);
        }
        item.active = req.state;
        try {
            await item.save();
            return response.api(201, item);
        }
        catch (error) {
            if (Config.get('app.debug')) {
                console.warn(`Error saving Repository Credentials State: ${error.toString()}`);
                console.warn(error);
            }
            return response.api(500, null, [{message: 'An error occured while trying to save this item. Please check with the system administrator.'}]);
        }
        return response.api(418, null, [{message: 'Debugging'}]);
    }
}

module.exports = RepositoryController
