'use strict'
const { validate, sanitize} = use('Validator');
const Webhook = use('App/Models/Webhook');
const Config = use('Config');

class WebhookController {
  async store({request, response, params}) {
    const rules = {
      name: 'required|string',
      active: 'required|boolean',
      url: 'required|url',
      method: 'required|in:get,post,put,patch,delete,copy,head,options,link,unlink,purge,lock,unlock,propfind,view',
      validate_ssl: 'required|boolean',
      auth_username: 'string',
      auth_password: 'string',
      headers: 'string',
      body_type: 'required_when:method,post|required_when:method,put|required_when:method,patch|string|in:plain,query,json',
      body: 'required_when:method,post|required_when:method,put|required_when:method,patch|string',
    };
    const transformations = {
      active: 'toBoolean',
      name: 'trim',
      url: 'strip_tags',
      method: 'trim',
      validate_ssl: 'toBoolean',
      auth_username: 'trim',
      auth_password: 'trim',
      headers: 'trim',
      body_type: 'trim',
      body: 'trim',
    }
    const messages = {
      'active.required': 'You must pass an activation state',
      'active.boolean': 'The activation state must be a boolean',
      'name.required': 'You must provide a name for the webhook',
      'name.string': 'You must provide a valid name for the webhook',
      'url.required': 'You must provide a URL for the webhook',
      'url.url': 'You must provide a valid URL for the webhook',
      'method.required': 'You must provide the webhooks HTTP Method',
      'method.in': 'The HTTP Method you specified is not supported',
      'validate_ssl.required': 'You must choose if to validate SSL or not',
      'validate_ssl.boolean': 'You must choose true or false if to validate SSL or not',
      'auth_username.string': 'Please pass a valid value for the HTTP Basic User',
      'auth_password.string': 'Please pass a valid value for the HTTP Basic Password',
      'headers': 'Please pass the headers as a line-seperated string',
      'body_type.required_when': 'You must choose a body type for this HTTP Method',
      'body_type.string': 'The body type must be Plain Text, URL Encoded Query or a JSON Object',
      'body_type.in': 'The body type must be Plain Text, URL Encoded Query or a JSON Object',
      'body.required_when': 'This HTTP Method requires a body',
      'body.string': 'Please pass the body as a string',
    }
    const validation = await validate(request.all(), rules, messages);
    if (validation.fails()) {
        return response.api(400, null, validation.messages());
    }
    const req = sanitize(request.all(), transformations);
    if ('json' == req.body_type) {
      try {
        JSON.parse(req.body);
      }
      catch (error) {
        return response.api(400, null, [{message: 'You have an error with your JSON object. Please check for errors and try again.'}]);
      }
    }
    let item;
    if (!params.wid) {
        item = new Webhook();
    }
    else {
        try {
            item = await Webhook.findOrFail(params.wid);
        }
        catch (error) {
            return response.api(500, null, [{message: 'An error occured while attempting to update the specified Webhook'}]);
        }
    }
    item.name = req.name;
    item.active = req.active;
    item.gitlab_repository_id = parseInt(params.id);
    item.repository_credentials_id = parseInt(params.cid);
    item.url = req.url;
    item.method = req.method;
    item.validate_ssl = req.validate_ssl;
    item.auth_username = req.auth_username;
    item.auth_password = req.auth_password;
    item.headers = req.headers;
    item.body_type = req.body_type;
    item.body = req.body;
    try {
      await item.save();
      return response.api(201, item);
    }
    catch (error) {
        if (Config.get('app.debug')) {
            console.warn(`Error saving Webhook: ${error.toString()}`);
            console.warn(error);
        }
        return response.api(500, null, [{message: 'An error occured while trying to save this item. Please check with the system administrator.'}]);
    }
  }

  async view({response, params}) {
    try {
      const item = await Webhook.findOrFail(params.wid);
      if (item.gitlab_repository_id !== parseInt(params.id)) {
          return response.api(404, null, [{message: 'The specified webhook do not exist for this repository'}]);
      }
      if (parseInt(params.cid) > 0) {
        if (item.repository_credentials_id !== parseInt(params.cid)) {
          return response.api(404, null, [{message: 'The specified webhook is not configured for these repository credentials'}]);
        }
      }
      else {
        if (parseInt(item.repository_credentials_id) > 0) {
          return response.api(404, null, [{message: 'The specified webhook is not a general webhook'}]);
        }
      }
      const ret = item.toJSON();
      return response.api(200, ret);
    }
    catch (error) {
        if (true == Config.get('app.debug')) {
            console.warn(`Error retrieving credentials: ${error.toString()}`);
            console.warn(error);
        }
        return response.api(500, null, [{message: 'An error occured while attempting to retrieve the specified Webhook'}]);
    }
    // return response.api(418, null, []);
  }

  async list({request, response, params}) {
    const query = Webhook.query().where('gitlab_repository_id', params.id);
    if (parseInt(params.cid) > 0) {
      query.where('repository_credentials_id', params.cid);
    }
    else {
      query.whereNull('repository_credentials_id');
    }
    const search = request.input('search');
    const options = request.input('options');
    let page = 1;
    let itemsPerPage = 10;
    if ('string' === typeof(search) && search.length > 0) {
        query.where(function() {
          this.where('name', 'like', `%${search}%`)
          this.orWhere('url', 'like', `%${search}%`)
        });
    }
    if ('object' == typeof(options)) {
        page = options.page;
        itemsPerPage = options.itemsPerPage;
        if (Array.isArray(options.sortBy)) {
            for (let i = 0; i < options.sortBy.length; i++) {
                const key = options.sortBy[i];
                const direction = (Array.isArray(options.sortDesc) && options.sortDesc.indexOf(key) > -1) ? 'desc' : 'asc';
                query.orderBy(key, direction);
            }
        }
        if (Array.isArray(options.groupBy)) {
            for (let i = 0; i < options.groupBy.length; i++) {
                const key = options.groupBy[i];
                query.groupBy(key);
            }
            if (['pg'].indexOf(Config.get('database.connection')) > -1) {
                query.groupBy('id');
            }
        }
    }
    const results = await query.paginate(page, itemsPerPage);
    const ret = results.toJSON();
    ret.data = ret.data.map((item) => {
        const ni = {};
        ni.id = item.id;
        ni.active = (true == item.active);
        ni.name = item.name;
        ni.url = item.url
        ni.method = item.method;
        return ni;
    })
    return response.api(200, ret);
  }

  async setActivation({request, response, params}) {
    const rules = {
      state: 'required|boolean',
    };
    const transformations = {
        state: 'toBoolean',
    }
    const messages = {
        'state.required': 'You must pass an activation state',
        'state.boolean': 'The activation state must be a boolean',
    }
    const validation = await validate(request.all(), rules, messages);
    if (validation.fails()) {
        return response.api(400, null, validation.messages());
    }
    const req = sanitize(request.all(), transformations);
    try {
      const item = await Webhook.findOrFail(params.wid);
      if (item.gitlab_repository_id !== parseInt(params.id)) {
          return response.api(404, null, [{message: 'The specified webhook do not exist for this repository'}]);
      }
      if (parseInt(params.cid) > 0) {
        if (item.repository_credentials_id !== parseInt(params.cid)) {
          return response.api(404, null, [{message: 'The specified webhook is not configured for these repository credentials'}]);
        }
      }
      else {
        if (parseInt(item.repository_credentials_id) > 0) {
          return response.api(404, null, [{message: 'The specified webhook is not a general webhook'}]);
        }
      }
      item.active = req.state;
      await item.save();
      return response.api(201, item);
      // const ret = item.toJSON();
      // return response.api(200, ret);
    }
    catch (error) {
        if (true == Config.get('app.debug')) {
            console.warn(`Error updating webhook: ${error.toString()}`);
            console.warn(error);
        }
        return response.api(500, null, [{message: 'An error occured while attempting to update the specified Webhook'}]);
    }
    return response.api(418, null, []);
  }
}

module.exports = WebhookController