const axios = require('axios');
const qs = require('qs');

class GitLab {
    constructor(host, client_id, client_secret, debug) {
        this.host = host;
        this.client_id = client_id;
        this.client_secret = client_secret;
        this.oauth = {};
        this.axios = axios.create({
            baseURL: `${host}/api/v4`,
            timeout: 59000,
            defaults: {
                headers: {
                    post: {
                        'Content-Type': 'application/json',
                    },
                    put: {
                        'Content-Type': 'application/json',
                    },
                    patch: {
                        'Content-Type': 'application/json',
                    },
                }
            }
        });
        this.token = null;
        this.auth = 'oauth';
        this.debug = debug;
    }

    setOauth(oauth) {
        if ('object' == typeof(oauth) && null !== oauth) {
            this.oauth = oauth;
            return true;
        }
        return false;
    }

    setToken(token) {
        if ('string' == typeof(token) && token.length > 0) {
            this.token = token;
            return true;
        }
        return false;
    }

    setAuthMethod(method) {
        if (['oauth', 'pat'].indexOf(method) > -1) {
            this.auth = method;
            return true;
        }
        return false;
    }

    handleError(error, response) {
        if (this.debug) {
            console.log('------------------------ Gitlab API Debug ------------------------');
            if ('undefined' !== typeof(error.response)) {
                console.log(`Method: ${error.request.method}`);
                console.log(`URL: ${error.config.url}`);
                console.log(`Headers: ${JSON.stringify(error.config.headers, null, 2)}`);
                if (['POST', 'PUT', 'PATCH'].indexOf(error.request.method) > -1) {
                    console.log(`Body: ${JSON.stringify(error.config.data, null, 2)}`);
                }
                console.log(`Response: ${JSON.stringify(error.response.data, null, 2)}`);
            }
            else {
                console.warn(error.toString());
                console.log(error);
            }
            console.log('-------------------------End of API Debug ------------------------');
        }
        if ('undefined' !== typeof(error.response)) {
            return response.api(error.response.status, null, [{message: error.response.data.error_description}]);
        }
        else {
            return response.api(500, null, [{message: 'An unknown error occured. Please contact a site administrator.'}])
        }
    }

    async getAccessToken(params) {
        return await this.axios.post(`${this.host}/oauth/token`, JSON.stringify(params), {
            headers: {
                'Content-Type': 'application/json',
            }
        });
    }

    async request(config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('oauth' == this.auth && 'object' == typeof(this.oauth) && null !== this.oauth && 'string' == typeof(this.oauth.access_token) ) {
            config.headers.Authorization = `Bearer ${this.oauth.access_token}`;
        }
        else if ('pat' == this.auth && 'string' == typeof(this.token)) {
            config.headers['Private-Token'] = this.token;
            config.headers['private-token'] = this.token;
            config.headers['X-Private-Token'] = this.token;
            config.headers['x-private-token'] = this.token;
        }
        if (this.debug) {
            console.log('------------------------ Gitlab API Debug ------------------------');
            console.log(`Method: ${config.method}`);
            console.log(`URL: ${config.url}`);
            console.log(`Headers: ${JSON.stringify(config.headers, null, 2)}`);
            if (['POST', 'PUT', 'PATCH'].indexOf(config.method) > -1) {
                console.log(`Body: ${JSON.stringify(config.data, null, 2)}`);
            }
            console.log('-------------------------End of API Debug ------------------------');
        }
        return await this.axios.request(config);
    }

    async get(url, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('oauth' == this.auth && 'object' == typeof(this.oauth) && null !== this.oauth && 'string' == typeof(this.oauth.access_token) ) {
            config.headers.Authorization = `Bearer ${this.oauth.access_token}`;
        }
        else if ('pat' == this.auth && 'string' == typeof(this.token)) {
            config.headers['Private-Token'] = this.token;
            config.headers['private-token'] = this.token;
            config.headers['X-Private-Token'] = this.token;
            config.headers['x-private-token'] = this.token;
        }
        if (this.debug) {
            console.log('------------------------ Gitlab API Debug ------------------------');
            console.log(`Method: GET`);
            console.log(`URL: ${url}`);
            console.log(`Headers: ${JSON.stringify(config.headers, null, 2)}`);
            console.log('-------------------------End of API Debug ------------------------');
        }
        return await this.axios.get(url, config);
    }

    async delete(url, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('oauth' == this.auth && 'object' == typeof(this.oauth) && null !== this.oauth && 'string' == typeof(this.oauth.access_token) ) {
            config.headers.Authorization = `Bearer ${this.oauth.access_token}`;
        }
        else if ('pat' == this.auth && 'string' == typeof(this.token)) {
            config.headers['Private-Token'] = this.token;
            config.headers['private-token'] = this.token;
            config.headers['X-Private-Token'] = this.token;
            config.headers['x-private-token'] = this.token;
        }
        if (this.debug) {
            console.log('------------------------ Gitlab API Debug ------------------------');
            console.log(`Method: DELETE`);
            console.log(`URL: ${url}`);
            console.log(`Headers: ${JSON.stringify(config.headers, null, 2)}`);
            console.log('-------------------------End of API Debug ------------------------');
        }
        return await this.axios.delete(url, config);
    }

    async head(url, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('oauth' == this.auth && 'object' == typeof(this.oauth) && null !== this.oauth && 'string' == typeof(this.oauth.access_token) ) {
            config.headers.Authorization = `Bearer ${this.oauth.access_token}`;
        }
        else if ('pat' == this.auth && 'string' == typeof(this.token)) {
            config.headers['Private-Token'] = this.token;
            config.headers['private-token'] = this.token;
            config.headers['X-Private-Token'] = this.token;
            config.headers['x-private-token'] = this.token;
        }
        if (this.debug) {
            console.log('------------------------ Gitlab API Debug ------------------------');
            console.log(`Method: HEAD`);
            console.log(`URL: ${url}`);
            console.log(`Headers: ${JSON.stringify(config.headers, null, 2)}`);
            console.log('-------------------------End of API Debug ------------------------');
        }
        return await this.axios.head(url, config);
    }

    async options(url, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('oauth' == this.auth && 'object' == typeof(this.oauth) && null !== this.oauth && 'string' == typeof(this.oauth.access_token) ) {
            config.headers.Authorization = `Bearer ${this.oauth.access_token}`;
        }
        else if ('pat' == this.auth && 'string' == typeof(this.token)) {
            config.headers['Private-Token'] = this.token;
            config.headers['private-token'] = this.token;
            config.headers['X-Private-Token'] = this.token;
            config.headers['x-private-token'] = this.token;
        }
        if (this.debug) {
            console.log('------------------------ Gitlab API Debug ------------------------');
            console.log(`Method: OPTIONS`);
            console.log(`URL: ${url}`);
            console.log(`Headers: ${JSON.stringify(config.headers, null, 2)}`);
            console.log('-------------------------End of API Debug ------------------------');
        }
        return await this.axios.options(url, config);
    }

    async post(url, data, config) {
        data = JSON.stringify(data);
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('oauth' == this.auth && 'object' == typeof(this.oauth) && null !== this.oauth && 'string' == typeof(this.oauth.access_token) ) {
            config.headers.Authorization = `Bearer ${this.oauth.access_token}`;
        }
        else if ('pat' == this.auth && 'string' == typeof(this.token)) {
            config.headers['Private-Token'] = this.token;
            config.headers['private-token'] = this.token;
            config.headers['X-Private-Token'] = this.token;
            config.headers['x-private-token'] = this.token;
        }
        if (this.debug) {
            console.log('------------------------ Gitlab API Debug ------------------------');
            console.log(`Method: POST`);
            console.log(`URL: ${url}`);
            console.log(`Headers: ${JSON.stringify(config.headers, null, 2)}`);
            console.log(`Body: ${JSON.stringify(data, null, 2)}`);
            console.log('-------------------------End of API Debug ------------------------');
        }
        return await this.axios.post(url, data, config);
    }

    async put(url, data, config) {
        data = JSON.stringify(data);
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('oauth' == this.auth && 'object' == typeof(this.oauth) && null !== this.oauth && 'string' == typeof(this.oauth.access_token) ) {
            config.headers.Authorization = `Bearer ${this.oauth.access_token}`;
        }
        else if ('pat' == this.auth && 'string' == typeof(this.token)) {
            config.headers['Private-Token'] = this.token;
            config.headers['private-token'] = this.token;
            config.headers['X-Private-Token'] = this.token;
            config.headers['x-private-token'] = this.token;
        }
        if (this.debug) {
            console.log('------------------------ Gitlab API Debug ------------------------');
            console.log(`Method: PUT`);
            console.log(`URL: ${url}`);
            console.log(`Headers: ${JSON.stringify(config.headers, null, 2)}`);
            console.log(`Body: ${JSON.stringify(data, null, 2)}`);
            console.log('-------------------------End of API Debug ------------------------');
        }
        return await this.axios.put(url, data, config);
    }

    async patch(url, data, config) {
        data = JSON.stringify(data);
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('oauth' == this.auth && 'object' == typeof(this.oauth) && null !== this.oauth && 'string' == typeof(this.oauth.access_token) ) {
            config.headers.Authorization = `Bearer ${this.oauth.access_token}`;
        }
        else if ('pat' == this.auth && 'string' == typeof(this.token)) {
            config.headers['Private-Token'] = this.token;
            config.headers['private-token'] = this.token;
            config.headers['X-Private-Token'] = this.token;
            config.headers['x-private-token'] = this.token;
        }
        if (this.debug) {
            console.log('------------------------ Gitlab API Debug ------------------------');
            console.log(`Method: PATCH`);
            console.log(`URL: ${url}`);
            console.log(`Headers: ${JSON.stringify(config.headers, null, 2)}`);
            console.log(`Body: ${JSON.stringify(data, null, 2)}`);
            console.log('-------------------------End of API Debug ------------------------');
        }
        return await this.axios.patch(url, data, config);
    }

    async all(iterable) {
        return await this.axios.all(iterable);
    }

    async spread(callback) {
        return await this.axios.spread(callback);
    }
}

function getGitLab() {
    const Config = use('Config');
    const config = Config.get('gitlab');
    return new GitLab(config.host, config.app_id, config.app_secret, Config.get('app.debug'));
}

module.exports = getGitLab;