const axios = require('axios');
const io = require('socket.io-client');

class ApiInteractor {
    constructor(origin, Vue) {
        this.origin = origin;
        this.jwt = '';
        this.axios = axios.create({
            baseURL: `${origin}/api/v1`,
            timeout: 59000,
            onUploadProgress: (progressEvent) => {
                let percentCompleted = Math.floor((progressEvent.loaded * 100) / progressEvent.total);
                this._trigger('axios-progress');
            }
        });
        this.axios.interceptors.request.use((config) => {
            this._trigger('axios-request');
            return config;
        });
        this.axios.interceptors.response.use((config) => {
            this._trigger('axios-response');
            return config;
        }, (error) => {
            this._trigger('axios-error');
            return Promise.reject(error);
        });
        this.socket = null;
        this._eventCallbacks = {
            setJwt: [],
            autologout: [],
            'axios-request': [],
            'axios-progress': [],
            'axios-response': [],
            'axios-error': [],
            'console.error': [],
        }
        this._socketCallbacks = new Map();
        this._initSocket();
        this.data = new Vue({
            data() {
                return {
                    me: false,
                }
            }
        })
    }

    on(event, callback) {
        if (Array.isArray(this._eventCallbacks[event])) {
            this._eventCallbacks[event].push(callback);
        }
    }

    socketOn(event, callback) {
        if (!this._socketCallbacks.has(event)) {
            this._socketCallbacks.set(event, []);
        }
        this._socketCallbacks.get(event).push(callback);
        if ('object' === typeof(this.socket) && null !== this.socket && 'function' === typeof(this.socket.disconnect)) {
            this.socket.on(event, callback);
        }
    }

    socketOff(event) {
        this._socketCallbacks.delete(event);
        if ('object' === typeof(this.socket) && null !== this.socket && 'function' === typeof(this.socket.disconnect)) {
            this.socket.off(event);
        }
    }

    setJwt(jwt) {
        if ('string' !== typeof(jwt)) {
            jwt = '';
        }
        this.jwt = jwt;
        this._initSocket();
        if (Array.isArray(this._eventCallbacks.setJwt)) {
            for (let i = 0; i < this._eventCallbacks.setJwt.length; i++) {
                const cb = this._eventCallbacks.setJwt[i];
                cb(this.jwt);
            }
        }
        return true;
    }

    _initSocket() {
        if ('object' === typeof(this.socket) && null !== this.socket && 'function' === typeof(this.socket.disconnect)) {
            this._socketCallbacks.forEach((cb, event) => {
                this.socket.off(event);
            });
            this.socket.off('error');
            this.socket.disconnect();
        }
        this.socket = io(`${this.origin}/admin`, {
            query: {
                jwt: this.jwt,
            }
        });
        this.socket.on('error', (error) => {
            if ('Invalid or Missing JWT' == error) {
                if ('string' === typeof(this.jwt) && this.jwt.length > 0) {
                    this.setJwt('');
                    this._trigger('autologout');
                }
            }
        });
        this._socketCallbacks.forEach((cb, event) => {
            this.socket.on(event, cb);
        });
    }

    _trigger(event, data) {
        if (Array.isArray(this._eventCallbacks[event])) {
            for (let i = 0; i < this._eventCallbacks[event].length; i++) {
                const cb = this._eventCallbacks[event][i];
                cb(data);
            }
        }
    }

    _handleAutoLogout(error) {
        if ('string' == typeof(error) && error.includes('You must be logged in')) {
            this.setJwt('');
            this._trigger('autologout');
        }
    }

    handleError(error) {
        if ('undefined' !== typeof(error.response)) {
            if (Array.isArray(error.response.data.errors)) {
                for (let i = 0; i < error.response.data.errors.length; i++) {
                    const err = error.response.data.errors[i];
                    switch (typeof (err)) {
                        case 'object':
                            this._handleAutoLogout(err.message);
                            this._trigger('console.error', err.message);
                            break;
    
                        case 'string':
                            this._handleAutoLogout(err);
                            this._trigger('console.error', err);
                            break;
                    }
                }
            } else {
                this._handleAutoLogout(error.response.data.error.message);
                this._trigger('console.error', error.response.data.error.message);
            }
        } else {
            console.error(error);
            this._trigger('console.error', 'An unrecoverable error occured. Please contact a system administrator.');
        }
    }

    async request(config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('string' === typeof(this.jwt) && this.jwt.length > 0) {
            config.headers.Authorization = `Bearer ${this.jwt}`;
        }
        return await this.axios.request(config);
    }

    async get(url, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('string' === typeof(this.jwt) && this.jwt.length > 0) {
            config.headers.Authorization = `Bearer ${this.jwt}`;
        }
        return await this.axios.get(url, config);
    }

    async delete(url, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('string' === typeof(this.jwt) && this.jwt.length > 0) {
            config.headers.Authorization = `Bearer ${this.jwt}`;
        }
        return await this.axios.delete(url, config);
    }

    async head(url, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('string' === typeof(this.jwt) && this.jwt.length > 0) {
            config.headers.Authorization = `Bearer ${this.jwt}`;
        }
        return await this.axios.head(url, config);
    }

    async options(url, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('string' === typeof(this.jwt) && this.jwt.length > 0) {
            config.headers.Authorization = `Bearer ${this.jwt}`;
        }
        return await this.axios.options(url, config);
    }

    async post(url, data, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('string' === typeof(this.jwt) && this.jwt.length > 0) {
            config.headers.Authorization = `Bearer ${this.jwt}`;
        }
        return await this.axios.post(url, data, config);
    }

    async put(url, data, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('string' === typeof(this.jwt) && this.jwt.length > 0) {
            config.headers.Authorization = `Bearer ${this.jwt}`;
        }
        return await this.axios.put(url, data, config);
    }

    async patch(url, data, config) {
        if ('object' !== typeof(config) || null == config) {
            config = {
                headers: {}
            };
        }
        if ('string' === typeof(this.jwt) && this.jwt.length > 0) {
            config.headers.Authorization = `Bearer ${this.jwt}`;
        }
        return await this.axios.patch(url, data, config);
    }

    async all(iterable) {
        return await this.axios.all(iterable);
    }

    async spread(callback) {
        return await this.axios.spread(callback);
    }
}

export default ApiInteractor;